[inherit('psqlite', 'sqlite')]
program test(input, output);

procedure checkcon(con : mysql_ptr);

begin
   if con = 0 then begin
      writeln(pmysql_error(con));
      halt;
   end;
end;

procedure checkstmt(stmt : mysql_stmt_ptr; con : mysql_ptr);

begin
   if stmt = 0 then begin
      writeln(pmysql_error(con));
      halt;
   end;
end;

procedure checkstat(stat : integer; stmt : mysql_stmt_ptr);

begin
   if stat <> 0 then begin
      writeln(pmysql_stmt_error(stmt));
      halt;
   end;
end;

function connect(host, un, pw, db : pstr) : mysql_ptr;

var
   con : mysql_ptr;

begin
   con := pmysql_init;
   checkcon(con);
   con := pmysql_real_connect(con, host, un, pw, db);
   checkcon(con);
   connect := con;
end;

procedure test_getone(con : mysql_ptr; f1 : integer);

var
   stmt : mysql_stmt_ptr;
   stat : integer;
   inparam : array[1..1] of mysql_bind;
   outparam : array[1..1] of mysql_bind;
   f2 : longpstr(255);

begin
   stmt := pmysql_stmt_init(con);
   checkstmt(stmt, con);
   stat := pmysql_stmt_prepare(stmt, 'SELECT f2 FROM t1 WHERE f1 = ?');
   checkstat(stat, stmt);
   pmysql_init_bind_long(inparam[1], f1);
   stat := pmysql_stmt_bind_param(stmt, inparam);
   checkstat(stat, stmt);
   stat := pmysql_stmt_execute(stmt);
   checkstat(stat, stmt);
   pmysql_init_bind_string_out(outparam[1], f2);
   stat := pmysql_stmt_bind_result(stmt, outparam);
   checkstat(stat, stmt);
   stat := pmysql_stmt_store_result(stmt);
   checkstat(stat, stmt);
   if pmysql_stmt_fetch(stmt) = 0 then begin
      writeln(stdstr(f2));
   end;
   pmysql_stmt_free_result(stmt);
end;

procedure test_getall(con : mysql_ptr);           

var
   stmt : mysql_stmt_ptr;
   stat : integer;
   outparam : array[1..2] of mysql_bind;
   f1 : integer;
   f2 : longpstr(255);

begin
   stmt := pmysql_stmt_init(con);
   checkstmt(stmt, con);
   stat := pmysql_stmt_prepare(stmt, 'SELECT f1,f2 FROM t1');
   checkstat(stat, stmt);
   stat := pmysql_stmt_execute(stmt);
   checkstat(stat, stmt);
   pmysql_init_bind_long(outparam[1], f1);
   pmysql_init_bind_string_out(outparam[2], f2);
   stat := pmysql_stmt_bind_result(stmt, outparam);
   checkstat(stat, stmt);
   stat := pmysql_stmt_store_result(stmt);
   checkstat(stat, stmt);
   while pmysql_stmt_fetch(stmt) = 0 do begin
      writeln(f1, ' ', stdstr(f2));
   end;
   pmysql_stmt_free_result(stmt);
end;

procedure test_insert(con : mysql_ptr; f1 : integer; f2 : pstr);

var
   stmt : mysql_stmt_ptr;
   stat : integer;
   inparam : array[1..2] of mysql_bind;

begin
   stmt := pmysql_stmt_init(con);
   checkstmt(stmt, con);
   stat := pmysql_stmt_prepare(stmt, 'INSERT INTO t1 VALUES(?, ?)');
   checkstat(stat, stmt);
   pmysql_init_bind_long(inparam[1], f1);
   pmysql_init_bind_string_in(inparam[2], f2);
   stat := pmysql_stmt_bind_param(stmt, inparam);
   checkstat(stat, stmt);
   stat := pmysql_stmt_execute(stmt);
   checkstat(stat, stmt);
   writeln(pmysql_stmt_affected_rows(stmt), ' rows inserted');
   pmysql_stmt_free_result(stmt);
end;

procedure test_delete(con : mysql_ptr; f1 : integer);

var
   stmt : mysql_stmt_ptr;
   stat : integer;
   inparam : array[1..1] of mysql_bind;

begin
   stmt := pmysql_stmt_init(con);
   checkstmt(stmt, con);
   stat := pmysql_stmt_prepare(stmt, 'DELETE FROM t1 WHERE f1 = ?');
   checkstat(stat, stmt);
   pmysql_init_bind_long(inparam[1], f1);
   stat := pmysql_stmt_bind_param(stmt, inparam);
   checkstat(stat, stmt);
   stat := pmysql_stmt_execute(stmt);
   checkstat(stat, stmt);
   writeln(pmysql_stmt_affected_rows(stmt), ' rows deleted');
   pmysql_stmt_free_result(stmt);
end;

var
   con : mysql_ptr;

begin
   con := connect('localhost', 'root', 'hemmeligt', 'Test');
   test_getone(con, 2);
   test_getall(con);
   test_insert(con, 999, 'XXX');
   test_getall(con);
   test_delete(con, 999);
   test_getall(con);
   pmysql_close(con);
end.
